//+------------------------------------------------------------------+
//|                                                      wininet.mqh |
//|                                                      Dirk Hilger |
//|                  Basic code taken from "gregspinner" @ mql5.com  |
//|                                     https://www.stereotrader.net |
//+------------------------------------------------------------------+
#property copyright "Copyright (C) 2016 by Doerk (Dirk Hilger)"
#property link      "www.stereotrader.net"
#property strict
#property version   "1.01"

#define INTERNET_DEFAULT_FTP_PORT 21
#define INTERNET_SERVICE_FTP 1            
#define INTERNET_FLAG_PASSIVE 0x8000000   
#define INTERNET_SERVICE_HTTP 3
#define INTERNET_DEFAULT_HTTP_PORT 80
#define INTERNET_DEFAULT_HTTPS_PORT 443
#define INTERNET_OPEN_TYPE_PRECONFIG 0
#define INTERNET_FLAG_SECURE 0x00800000
#define INTERNET_FLAG_RELOAD 0x80000000

#define FTP_TRANSFER_TYPE_UNKNOWN 0x0   
#define FTP_TRANSFER_TYPE_ASCII 0x1   
#define FTP_TRANSFER_TYPE_BINARY 0x2  
#define INTERNET_FLAG_TRANSFER_UNKNOWN 0x0
#define INTERNET_FLAG_TRANSFER_ASCII 0x1
#define INTERNET_FLAG_TRANSFER_BINARY 0x2

#define GENERIC_WRITE 0x40000000
#define GENERIC_READ 0x80000000

enum ENUM_SSL_USAGE
{
   SSL_DEFAULT = 0,
   SSL_ON = 1,
   SSL_OFF = 2
};

#define LPCTSTR string
#define HINTERNET int 

#import  "wininet.dll"
   HINTERNET InternetOpenW(string name, int config, string, string, int);
   HINTERNET InternetConnectW(HINTERNET hInternet, string lpszServerName, int nServerPort, string lpszUsername, string lpszPassword, int dwService, int dwFlags, int dwContext);
   HINTERNET HttpOpenRequestW(HINTERNET hConnect, string lpszVerb, string lpszObjectName, string lpszVersion, string lpszReferer, string lpszAcceptTypes, int dwFlags, int dwContext);
   bool HttpSendRequestW(HINTERNET hRequest, string lpszHeaders, int dwHeadersLength, const uchar &lpOptional[], int dwOptionalLength);
   bool InternetReadFile(HINTERNET hRequest, uchar &sBuffer[], int dwNumberOfBytesToRead, int &lpdwNumberOfBytesRead);
   bool InternetCloseHandle(HINTERNET hInternet);
#import

#import "kernel32.dll"
   int GetLastError(void);
#import

string __wininet_connect = "StereoTrader_API";
bool __wininet_ssl = true;

// Function to set a unique connection ID
void InternetSetIDString(string id)
{
   __wininet_connect = "StereoTrader_API";
   if (id != NULL)
      __wininet_connect += "_" + id;
}

// Function to check and format the HTTP/HTTPS URL
string CheckHTTP(string url, ENUM_SSL_USAGE sslmode)
{
   if (StringFind(url, "http") < 0)
      url = "http://" + url;

   bool ssluse = __wininet_ssl;
   if (sslmode == SSL_OFF)
      ssluse = false;
   else if (sslmode == SSL_ON)
      ssluse = true;

   if (ssluse)
      StringReplace(url, "http://", "https://");
   else
      StringReplace(url, "https://", "http://");

   return url;
}

// Function to get HTTP content
int InternetGetHTTPContent(string url, string& content, ENUM_SSL_USAGE sslmode = SSL_DEFAULT)
{
   content = NULL;
   url = CheckHTTP(url, sslmode);

   HINTERNET hInternet = InternetOpenW(__wininet_connect, INTERNET_OPEN_TYPE_PRECONFIG, NULL, NULL, 0);
   if (hInternet == 0)
   {
      Print("Error opening internet connection. Error code: ", GetLastError());
      return GetLastError();
   }

   int port = (sslmode == SSL_ON) ? INTERNET_DEFAULT_HTTPS_PORT : INTERNET_DEFAULT_HTTP_PORT;
   string server = url;
   string path = "";  // Extract path from URL if necessary
   int pos = StringFind(url, "/", StringFind(url, "://") + 3);
   if (pos > 0)
   {
      server = StringSubstr(url, 0, pos);
      path = StringSubstr(url, pos);
   }

   HINTERNET hConnect = InternetConnectW(hInternet, server, port, NULL, NULL, INTERNET_SERVICE_HTTP, 0, 0);
   if (hConnect == 0)
   {
      Print("Error opening HTTP session. Error code: ", GetLastError());
      InternetCloseHandle(hInternet);
      return GetLastError();
   }

   uint flags = INTERNET_FLAG_RELOAD | (sslmode == SSL_ON ? INTERNET_FLAG_SECURE : 0);
   HINTERNET hRequest = HttpOpenRequestW(hConnect, "GET", path, NULL, NULL, NULL, flags, 0);
   if (hRequest == 0)
   {
      Print("Error opening HTTP request. Error code: ", GetLastError());
      InternetCloseHandle(hConnect);
      InternetCloseHandle(hInternet);
      return GetLastError();
   }

   uchar emptyOptional[];  // Use an empty array instead of NULL

   if (!HttpSendRequestW(hRequest, "", 0, emptyOptional, 0))
   {
      Print("Error sending HTTP request. Error code: ", GetLastError());
      InternetCloseHandle(hRequest);
      InternetCloseHandle(hConnect);
      InternetCloseHandle(hInternet);
      return GetLastError();
   }

   uchar buffer[1024];
   int bytesRead = 0;
   string tempResponse = "";
   while (InternetReadFile(hRequest, buffer, sizeof(buffer), bytesRead) && bytesRead > 0)
   {
      tempResponse += CharArrayToString(buffer, 0, bytesRead);
   }

   content = tempResponse;

   InternetCloseHandle(hRequest);
   InternetCloseHandle(hConnect);
   InternetCloseHandle(hInternet);

   return 0;  // Success
}

// Function to get HTTP content with optional cache disabling
string InternetGetHTTPContent(string url, bool autodisablecache = true, ENUM_SSL_USAGE sslmode = SSL_DEFAULT)
{
   string content = NULL;
   if (autodisablecache)
   {
      double rnd = ::GetTickCount();
      string rndstr = DoubleToString(rnd);
      if (StringFind(url, "?") >= 0)
         url = url + "&a=" + rndstr;
      else
         url = url + "?a=" + rndstr;
   }
   int e = InternetGetHTTPContent(url, content, sslmode);
   return content;
}

// Function to send POST request
bool PostHTTPRequest(string domain, string path, string postData, string& response, ENUM_SSL_USAGE sslmode = SSL_DEFAULT)
{
    string headers = "Content-Type: application/x-www-form-urlencoded\r\n";
    int port = (sslmode == SSL_ON) ? INTERNET_DEFAULT_HTTPS_PORT : INTERNET_DEFAULT_HTTP_PORT;

    HINTERNET hInternet = InternetOpenW(__wininet_connect, INTERNET_OPEN_TYPE_PRECONFIG, NULL, NULL, 0);
    if (hInternet == 0)
    {
        Print("Error opening internet connection. Error code: ", GetLastError());
        return false;
    }

    HINTERNET hConnect = InternetConnectW(hInternet, domain, port, NULL, NULL, INTERNET_SERVICE_HTTP, 0, 0);
    if (hConnect == 0)
    {
        Print("Error opening HTTP session. Error code: ", GetLastError());
        InternetCloseHandle(hInternet);
        return false;
    }

    uint flags = INTERNET_FLAG_RELOAD | ((sslmode == SSL_ON) ? INTERNET_FLAG_SECURE : 0);
    HINTERNET hRequest = HttpOpenRequestW(hConnect, "POST", path, NULL, NULL, NULL, flags, 0);
    if (hRequest == 0)
    {
        Print("Error opening HTTP request. Error code: ", GetLastError());
        InternetCloseHandle(hConnect);
        InternetCloseHandle(hInternet);
        return false;
    }

    uchar postDataArray[];
    StringToCharArray(postData, postDataArray, 0, StringLen(postData));

    if (!HttpSendRequestW(hRequest, headers, StringLen(headers), postDataArray, ArraySize(postDataArray)))
    {
        Print("Error sending HTTP request. Error code: ", GetLastError());
        InternetCloseHandle(hRequest);
        InternetCloseHandle(hConnect);
        InternetCloseHandle(hInternet);
        return false;
    }

    uchar buffer[1024];
    int bytesRead = 0;
    string tempResponse = "";
    while (InternetReadFile(hRequest, buffer, sizeof(buffer), bytesRead) && bytesRead > 0)
    {
        tempResponse += CharArrayToString(buffer, 0, bytesRead);
    }

    response = tempResponse;

    InternetCloseHandle(hRequest);
    InternetCloseHandle(hConnect);
    InternetCloseHandle(hInternet);

    return true;
}
